<?php
// $Id: functions.inc,v 1.46 2005/07/21 19:05:53 saschalorenz Exp $
/* Functions.inc: Includes all the functions needed by TFTGallery. The
 *                file is split up in serveral sections (e.g. URL-handling,
 *                helper-functions, etc.)
 */


/* BEGIN SECTION:   Constants
 *
 *        Define some constants.
 */

define ("TFT_VERSION", "0.10");
define ("TFT_INFO", 3);
define ("TFT_WARNING", 2);
define ("TFT_ERROR", 1);
 
/* END SECTION:   Constants
 */

/* BEGIN SECTION:   URL-handling
 *
 *        Functions to make handling of URLs easier.
 */

// setGlobal(): promote a request variable to global scope
// $name: variable name
function setGlobal($name,$option = false) {
	if (isset($_REQUEST[$name])) {
		$GLOBALS[$name] = $_REQUEST[$name];
	} elseif ($option!==false && isset($_REQUEST[$option])) {
		$GLOBALS[$name] = $_REQUEST[$option];
	}
}


// setupStickyURLOptions(): setup sticky options global variable
function setupStickyURLOptions() {
	global $sticky_options;
	$sticky_options_match = '/^(lang|tl|style|debug)$/i';
	$sticky_options = array();
	if (isset($_GET)) {
		foreach($_GET as $option => $value) {
			if (preg_match($sticky_options_match,$option)) {
				$sticky_options[$option] = $value;
			}
		}
	}
}


// setStickyOption(): set the value of a sticky option
// $option: option name
// $value: option value
function setStickyOption($option,$value = false) {
	global $sticky_options;
	$sticky_options[$option] = $value;
}


// unsetStickyOption(): clear the value of a sticky option
// $option: option name
function unsetStickyOption($option) {
	global $sticky_options;
	unset($sticky_options[$option]);
}


// makeURL(): properly format a url
// $page: base URL, may be a null string
// $option: [array] of option, keys are option name, values are option values
function makeURL($page,$params = array(),$no_sticky = false) {
	global $sticky_options;
	$url = "$page?";
	$first = true;
	// add all the params, encoded as needed
	foreach($params as $param => $value) {
		$url .= ($first ? "" : "&amp;") . "$param=" . urlencode($value);
		$first = false;
	}
	// add any sticky options
	if (!$no_sticky and is_array($sticky_options)) {
		foreach($sticky_options as $option => $value) {
			if (!isset($params[$option])) {
				$url .= ($first ? "" : "&amp;") . "$option=" . urlencode($value);
				$first = false;
			}
		}
	}
	return $url;
}

// makeHiddenVars(): generate a string of hidden variables for user params and sticky options
// $params: [array] of option, keys are option name, values are option values
function makeHiddenVars($params = array(),$no_sticky = false) {
	global $sticky_options;
	$output = '';
	
	// add all the params, encoded as needed
	foreach($params as $param => $value) {
		$output .= "<input type=\"hidden\" name=\"$param\" value=\"$value\" />\n";
	}
	// add any sticky options
	if (!$no_sticky) {
		foreach($sticky_options as $option => $value) {
			if (!isset($params[$option])) {
				$output .= "<input type=\"hidden\" name=\"$option\" value=\"$value\" />\n";
			}
		}
	}
	return $output;
}

/* END SECTION:   URL-handling
 */

/* BEGIN SECTION:   Settings
 *
 *        Functions to handle the settings of TFTGallery.
 */

// albumInclude(): detect and process album specific config files
// $album: relative album path
function albumInclude($album) {
	if (file_exists(makeAlbumPath($album) . "/album.inc")) {
		include_once(makeAlbumPath($album) . "/album.inc");
	}
}

// albumStyleSheet(): detect album specific css stylesheet
// $album: relative album path
function albumStyleSheet($album) {
	$sspath = makeAlbumPath($album) . "/album.css";
	if (file_exists($sspath)) {
		return $sspath;
	}
	return false;
}

// getLanguageInclude(): returns the inc-file for the right language
// $language: the language requested by the browser
function getLanguageInclude($language) {
	// setup browser requested language
	// default
	$lang_inc = "strings.$language.inc";
	// url over-ride
	if (isset($_REQUEST['lang']) && file_exists("language/strings.${_REQUEST['lang']}.inc")) {
		$lang_inc = "strings." . $_REQUEST['lang'] . ".inc";
	}
	// personal setting
	elseif (isset($_REQUEST['personal_language']) && file_exists("language/strings.${_REQUEST['personal_language']}.inc")) {
		$lang_inc = "strings.${_REQUEST['personal_language']}.inc";
	} 
	// browser setting
	elseif (isset($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
		$requested_lang_inc = "strings." . substr($_SERVER['HTTP_ACCEPT_LANGUAGE'],0,2) . ".inc";
		if (file_exists("language/" . $requested_lang_inc)) {
			$lang_inc = $requested_lang_inc;
		}
	}
	return $lang_inc;
}

// checkAlbumParamter(): verify that album param refers to a folder that exists
function checkAlbumParamter(){
	global $album_path;
	$folder = $album_path;
	return is_dir($folder) ? true : false;
}
/* END SECTION:   Settings
 */

/* BEGIN SECTION:   File-handling
 *
 *        Functions to work with files and directories
 */

// getPictureList() returns list of all files in a directory or virtual directory
// $album: relative path to album
function getPictureList($album){
	$dir = makeAlbumPath($album);
//echo "### $dir " . file_exists("$dir/virtual.txt") . "<br/>\n";
	if (file_exists("$dir/virtual.txt")) {
		$lines = file("$dir/virtual.txt");
		$list = array();
		foreach($lines as $line) {
			if (substr($line,0,1)=='#') {
				$voption = substr($line,1);
			} elseif (trim($line)!=="") {
				$line = trim($line);
				if (preg_match("/^-([^\-0-9]*)([0-9]+)([^\-0-9]+)-([^\-0-9]*)([0-9]+)([^\-0-9]*)$/",$line,$tag)) {
					$width = strlen($tag[2]);
					$prefix = $tag[1];
					$begin = 1 * $tag[2];
					$postfix = $tag[3];
					$end = 1 * $tag[5];
					//echo "width = $width<br/>\n";
					//echo "prefix = $prefix<br/>\n";
					//echo "postfix = $postfix<br/>\n";
					//echo "begin = $begin<br/>\n";
					//echo "end = $end<br/>\n";
					if ($begin!=0 && $end!=0 && $begin<$end) {
						for ($i = $begin; $i <= $end; $i++) {
							$num = "0000000000000000" . $i;
							$file = $prefix . substr($num,strlen($num)-$width) . $postfix;
							if (file_exists($file)) {
								$list[] = $file;
							}
						}
					}
				} else {
					$list[] = trim($line);
				}
			} else {
				$list[] = "zzzz";
			}
		}
		reset($list);
	} else {
		$list = getFilelist($album);
	}
	return $list;
}

function makeAlbumPath($album) {
	global $album_directory;
	return $album=="" ? $album_directory : "$album_directory/$album";
}

// getFilelist() returns list of all files (relative paths from $data_directory)in a directory
// $album: relative path to album directory
function getFilelist($album){
	global $pic_extentions, $other_extentions;
	$dir = makeAlbumPath($album);
	$list = array();
//echo "'$dir'";
	$fd = opendir($dir);
	$i = 0;
	while($file = readdir($fd)){
		// skip files and directories begining with .
		if(substr($file,0,1)=='.')
		{
			continue;
		}
		if(is_file("$dir/$file") && (preg_match($pic_extentions,$file) || preg_match($other_extentions,$file))) {
			$list[$i] = $file;
			$i++;
		}
		elseif(is_dir("$dir/$file"))
		{
			$list[$i] = $file."/";
			$i++;
		}
		
	}
	closedir($fd);
	sort($list, SORT_STRING);
	reset($list);
	return $list;
}

// makeDataPath($dest): create a data directory recursively
// $dest: destination directory
function makeDataPath($dest)
{
	global $data_directory;
	// create folders as needed
	$elements = explode("/", $dest);
	$cwd = getcwd();
	// print "cwd=$cwd<br>\n";
	chdir($data_directory);
	// $cur=getcwd();
	for($i = 0; $i < (count($elements)); $i++){
	// $cur=getcwd();
	// print "$cur -- making=".$elements[$i]."<br>\n";
		@mkdir($elements[$i], 0777);
		@chmod($elements[$i], 0777);
		chdir($elements[$i]);
	}
	chdir($cwd);
}

// getThumbnailDescription($album,$pic): get any description
// $album: album
// $pic: picture
function getThumbnailDescription($album,$pic){
	global $album_directory,$data_directory;
	$desc = "$album_directory/$album/$pic";
	if(file_exists("$desc.txt")) {
		return(htmlentities(file_get_contents("$desc.txt")));
	} 
	elseif(file_exists("$desc.html")) {
		return(file_get_contents("$desc.html"));
	}
	elseif(file_exists("$data_directory/description/$album/$pic.txt")) {
		return(htmlentities(file_get_contents("$data_directory/description/$album/$pic.txt")));
	}
	elseif(file_exists("$data_directory/description/$album/$pic.html")) {
		return(file_get_contents("$data_directory/description/$album/$pic.html"));
	}
}

// showThumbnailDescription($description): 
// $description: text
function showThumbnailDescription($description){
	if(!empty($description))
	{
		echo "<div class='picture_description'>\n";
		echo $description;
		echo "</div>\n";
	}
}



/* END SECTION:   File-handling
 */
 
/* BEGIN SECTION:   Server Scaling-functions
 *
 *        Some small functions to do the dirty work...
 */


// validScaledFileExists(): determins if the picture already has an existing scaled
// $pic: relative path to picture
// $size: 1 - thumbnail, 2 - scaled
function validScaledFileExists($pic,$size = 1){
	global $album_directory, $gd_available;
	$scaled_time = 0;
	$scaled_file_path = getScaledFilePath($pic,$size);
	$pic_path = "$album_directory/$pic";
	@$scaled_time = filemtime($scaled_file_path);
	@$pic_time = filemtime($pic_path);
	// compare mod dates of picture and scaled
	// use out-of-date scaled file if GD is not available
	if (function_exists('logh')) {
		logh("Scaled_file: '$scaled_file_path' $scaled_time Pic: '$pic_path' $pic_time");
	}
	if ($scaled_time!==FALSE && (!$gd_available || $pic_time<$scaled_time)) {
		$result = true;
	} else {
		$result = false;
	}
	return $result;
}

// getScaledFilePath($pic): generate proper scaled file path
// $pic: relative path to picture 
// $size: 1 - thumbnail, 2 - scaled
function getScaledFilePath($pic, $size = 1){
	global $data_directory, $use_short_names;
	$folder = $size==1 ? "thumbs" : "scaled";
	$ext = $use_short_names ? (preg_match("/\.(jpg|jpeg)/i",$pic) ? "" : ".jpg") : ".jpg";
	return "$data_directory/$folder/${pic}$ext";
}

// getPicturePath($pic): generate proper scaled file path
// $pic: path to picture relative to $album_directory
function getPicturePath($pic) {
	global $album_directory;
	return "$album_directory/$pic";
}

// @@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@@@@@ TO BE DELETED @@@@@@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@
// validScaledExists(): determins if the picture already has an existing scaled
// $pic: relative path to picture
// $size: 1 - thumbnail, 2 - scaled
function validScaledExists($pic,$size = 1) {
	global $gd_available;
	$scaled_time = 0;
	$tn_file_name = getScaledURL($pic,$size);
	@$scaled_time = filemtime($tn_file_name);
	@$pic_time = filemtime($pic);
	// compare mod dates of picture and scaled
	// use out-of-date scaleds if GD is not available
	if ($scaled_time!==FALSE && (!$gd_available || $pic_time<$scaled_time)) {
		$result = true;
	} else {
		$result = false;
	}
	return $result;
}

// scaleWidthHeight() scale width & heigh to fit in target width and height
// $width - original width
// $width - original height
// $target_x - target width
// $target_y - target height
function scaleWidthHeight($width,$height,$target_x,$target_y) {
	$new_width=$new_height=0;
	if ($width==0 || $height==0) {
		$new_x = $target_x;
		$new_y = $target_y;
	} elseif ($width<=$target_x && $height<=$target_y) {
		$new_x = $width;
		$new_y = $height;
	} elseif (($width / $height) > ($target_x / $target_y)){
		$new_width = $target_x;
		$new_height = ($target_x / $width) * $height;
	}
	else {
		$new_height = $target_y;
		$new_width = ($target_y / $height) * $width;
	}
	return array(round($new_width),round($new_height));
}

// makeScaledFileURL() makes URL to reference or create proper size image
// $pic: path to picture relative to $album_directory
// $size: 1 - thumbnail, 2 - scaled
// returns list($url,$new_width,$new_height,$old_width,$old_height,$is_picture,$is_icon)
function makeScaledFileURL($pic,$size) {
	global $max_picture_width, $max_picture_height, $max_picture_scaling, $thumb_width, $thumb_height, $thumb_map, $other_extentions, $pic_extentions, $gd_available;
	$pic_path = getPicturePath($pic);
	$url = $pic_path;
	if (!preg_match($pic_extentions,$pic)) {
		$is_icon = true;
		$is_picture = false;
		$url = "images/default_error.gif";
		if(preg_match($other_extentions,$pic)){
			$url = isset($thumb_map['default']) ? $thumb_map['default'] : "images/default_unknown.gif";
			foreach($thumb_map as $pat => $filename) {
				if (strpos($pat,"/")!==false && preg_match($pat,$pic)) {
					$url = "$filename";
					break;
				}
			}
		}
		$elements = getimagesize($url);
		$width = $x = $elements[0];
		$height = $y = $elements[1];
	} else {
		$is_icon = false;
		$is_picture = true;
		$scaled_path = getScaledFilePath($pic,$size);
		$elements = getimagesize($pic_path);
		if ($elements===false) {
			$no_size = true;
			$img_type = false;
			$x = $y = 0;
		} else {
			$no_size = false;
			$x = $elements[0];
			$y = $elements[1];
			$img_type = $elements[2]>=1 && $elements[1]<=3;
		}
		$width = $x;
		$height = $y;
		// assume using original
		$url = $pic_path;
		if ($size==1) {
			// assume using original
			$url = $pic_path;
			// too big?
			if ($no_size || !$img_type || $x>$thumb_width || $y>$thumb_height) {
				// use existing?
				if (validScaledFileExists($pic,$size)) {
					$url = $scaled_path; 
					list($width,$height,$yadyada) = getimagesize($scaled_path);
				}
				// generate
				elseif ($gd_available) {
					$url = makeURL("scaler.php",array("picture" => $pic, "size" => $size),true);
					if (!$no_size) {
						list($width,$height) = scaleWidthHeight($x,$y,$thumb_width,$thumb_height);
					}
				} else {
					// force client side scaling for thumbnails
					list($width,$height) = scaleWidthHeight($x,$y,$thumb_width,$thumb_height);
				}
			}
		} else {
			// too big?
			if ($no_size || !$img_type || $x*$y > $max_picture_width*$max_picture_height*$max_picture_scaling) {
				// use existing?
				if (validScaledFileExists($pic,$size)) {
					$url = $scaled_path;
					list($width,$height,$yadyada) = getimagesize($scaled_path);
				} 
				// generate
				elseif ($gd_available) {
					$url = makeURL("scaler.php",array("picture" => $pic, "size" => $size),true);
					if (!$no_size) {
						list($width,$height) = scaleWidthHeight($x,$y,$max_picture_width,$max_picture_height);
					}
				} else {
					// force client side scaling for thumbnails
					list($width,$height) = scaleWidthHeight($x,$y,$max_picture_width,$max_picture_height);
				}
			}
		}
	}
	return array($url,$width,$height,$x,$y,$is_picture,$is_icon);
}

// @@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@@@@@ TO BE DELETED @@@@@@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@
// getScaledURL($pic): generate proper scaled file path
// $pic: relative path to picture 
// $size: 1 - thumbnail, 2 - scaled
function getScaledURL($pic, $size = 1) {
	global $data_directorym, $use_short_names;
	$folder = $size==1 ? "thumbs" : "scaled";
	$ext = $use_short_names ? (preg_match("/\.(jpg|jpeg)/i",$pic) ? "" : ".jpg") : ".jpg";
	return "$data_directory/$folder/${pic}$ext";
}

/* END SECTION:   Server Scaling
 */
 
/* BEGIN SECTION:   Helper-functions
 *
 *        Some small functions to do the dirty work...
 */

// reverse_strrchr(): returns the characters preceeding the last occurance of the needle
// $haystack: target
// $needle: search string
function reverse_strrchr($haystack, $needle) {
   $pos = strrpos($haystack, $needle);
   if($pos===false) {
       return $haystack;
   }
   return substr($haystack, 0, $pos + 1);
}


// pre-PHP5 compatibility
if (!function_exists('file_get_contents')) { 
	// file_get_contents(): reads the contents of a file into a string
	// $fielname: path to file
	// $use_include_path: search for relative paths along the include path (default false)
	function file_get_contents($filename, $use_include_path = 0) { 
		$file = @fopen($filename, 'rb', $use_include_path); 
		if ($file) { 
			if ($fsize = @filesize($filename)) { 
				$data = fread($file, $fsize); 
			}
			else { 
				while (!feof($file)) { 
				$data .= fread($file, 1024); 
				}
			} 
			fclose($file); 
		} 
		return $data; 
	} 
}

// d(): outout string if in debug mode
// $s string to output
function d($s) {
	global $debug;
	if ($debug) {
		echo $s . "<br/>\n";
	}
}

// setTextDefault(): 
// $name name of variable to set
// $value value to assign to variable with name $name
function setTextDefault($name,$value) {
	$GLOBALS[$name] = isset($GLOBALS[$name]) ? $GLOBALS[$name] : $value;
}

// log2file(): log a message of specified priority to the log file
// $message message to log
// $priority priority of message
function log2File($message, $priority){
	if($GLOBALS["log_level"] >= $priority){
		// compose the logstring:
		switch($priority){
			case TFT_ERROR:
				$logstring =  "(EE) ";
				break;

			case TFT_WARNING:
				$logstring =  "(WW) ";
				break;

			case TFT_INFO:
				$logstring =  "(II) ";
				break;

			default:
				return;
		}
		$logstring = $logstring . date("d.m.Y/H:i:s") . " " . $message . "\r\n";

		if($fh = @fopen("gallery.log", "a"))
		{
			fwrite($fh, $logstring);
			fclose($fh);
		}
		
	}
}
/* END SECTION:   Helper-functions
 */

/* BEGIN SECTION:   compatibility-functions
 *
 *        compatibility for older versions of PHP...
 */
if (!function_exists('file_get_contents'))
{
    function file_get_contents ($filename, $incpath = false, $resource_context = null)
    {
        if ($fh = @fopen($filename, 'rb', $incpath) === false) {
            trigger_error('file_get_contents() failed to open stream: No such file or directory', E_USER_WARNING);
            return false;
        }

        clearstatcache();
        if ($fsize = filesize($filename)) {
            $data = fread($fh, $fsize);
        }
        
        else {
            while (!feof($fh)) {
                $data .= fread($fh, 8192);
            }
        }

        fclose($fh);

        return $data;    
    }
}

if (!defined('FILE_USE_INCLUDE_PATH')) {
    define('FILE_USE_INCLUDE_PATH', 1);
}

if (!defined('FILE_APPEND')) {
    define('FILE_APPEND', 8);
}

if (!function_exists('file_put_contents'))
{
    function file_put_contents ($filename, $content, $flags = null, $resource_context = null)
    {
        // If $content is an array, convert it to a string
        if (is_array($content)) {
            $content = implode('', $content);
        }

        // If we don't have a string, throw an error
        if (!is_string($content)) {
            trigger_error('file_put_contents() The 2nd parameter should be either a string or an array', E_USER_WARNING);
            return false;
        }        
        
        // Get the length of date to write
        $length = strlen($content);

        // Check what mode we are using
        $mode = ($flags & FILE_APPEND) ?
                    $mode = 'a' :
                    $mode = 'w';

        // Check if we're using the include path
        $use_inc_path = ($flags & FILE_USE_INCLUDE_PATH) ?
                    true :
                    false;

        // Open the file for writing
        if (($fh = @fopen($filename, $mode, $use_inc_path)) === false) {
            trigger_error('file_put_contents() failed to open stream: Permission denied', E_USER_WARNING);
            return false;
        }

        // Write to the file
        $bytes = 0;
        if (($bytes = @fwrite($fh, $content)) === false) {
            $errormsg = sprintf('file_put_contents() Failed to write %d bytes to %s',
                            $length,
                            $filename);
            trigger_error($errormsg, E_USER_WARNING);
            return false;
        }

        // Close the handle
        @fclose($fh);

        // Check all the data was written
        if ($bytes != $length) {
            $errormsg = sprintf('file_put_contents() Only %d of %d bytes written, possibly out of free disk space.',
                            $bytes,
                            $length);
            trigger_error($errormsg, E_USER_WARNING);
            return false;
        }

        // Return length
        return $bytes;
    }
}

/* END SECTION:   compatibility-functions
 */



/* BEGIN SECTION:   scaler-functions
 *
 *        scale images
 */

// createScaled($pic): creates a scaled for an specified image and saves it.
// $pic: path to the picture
// $output: send jpeg to output
function createScaled($pic,$size = 1,$output = true, $force=false){
	global $gd_available, $data_directory, $album_directory, $thumb_width, $thumb_height, $max_picture_width, $max_picture_height, $max_picture_scaling;
	$dest = getScaledFilePath($pic,$size);
	if (function_exists('logh')) {
		logh("Create destination: $dest");
	}
	
	if ($size==1) {
		$x = $thumb_width;
		$y = $thumb_height;
	} elseif ($size==2) {
		$x = $max_picture_width;
		$y = $max_picture_height;
	}
	// create folders as needed
	$elements = explode("/", $dest);
	$cwd = getcwd();
	chdir($data_directory);
	for($i = 1; $i < (count($elements) - 1); $i++){
		@mkdir($elements[$i], 0777);
		@chmod($elements[$i], 0777);
		chdir($elements[$i]);
	}
	chdir($cwd);
	// calculate size of sized depending on _height/_width
	list($image_x, $image_y, $type, $attr) = getimagesize("$album_directory/$pic");
	if(($type < 1) || ($type > 3)) {
		if (function_exists('logh')) {
			logh("not a picture!");
		}
	return(false);
	}
	if (function_exists('logh')) {
		logh("Original size: $image_x by $image_y");
	}
	
	$factorx = $x / $image_x;
	$factory = $y / $image_y;
	$factor = ($factorx>$factory) ? $factory : $factorx;
	if($factor > $max_picture_scaling) $factor = $max_picture_scaling;
	$new_height = ceil($image_y * $factor);
	$new_width = ceil($image_x * $factor);
	
	if (function_exists('logh')) {
		logh("New size: $new_width by $new_height");
	}
	
	if ($gd_available>=1 && $gd_available<=9) {
		return(createScaled_GD($pic,$new_width,$new_height,$size,$output,$force));
	} elseif ($gd_available==10) {
		return(createScaled_IM($pic,$new_width,$new_height,$size,$output,$force));
	}
}

// createScaled($pic): creates a scaled for an specified image and saves it using ImageMagick.
// $pic: path to the picture
// $output: send jpeg to output
function createScaled_IM($pic,$new_width,$new_height,$size,$output = true, $force=false){
	global $data_directory, $album_directory, $im_path, $server_scaled_quality;
	$src = "$album_directory/$pic";
	$dest = getScaledFilePath($pic,$size);
	
	if (file_exists($dest))
	{
		if(filemtime($src) > filemtime($dest) or $force)
		{
			@unlink($dest);
		}
		else
		{
			return(false);
		}
	}
	
	$cmd = "$im_path " . escapeshellarg($src) . " -resize " . escapeshellarg("${new_width}x$new_height>") . " +profile \"*\" " .  escapeshellarg($dest);
	header("X-TFT-Error-IM-Cmd: $cmd"); 
	// execute command, force stderr to stdout
	$out = `$cmd 2>&1`;
	if (trim($out)=='' || file_exists($dest)) {
		if (isset($file_permissions) && $file_permissions!='') chmod($dest,$file_permissions);
		if ($output) readfile($dest);
	} else {
		header("X-TFT-Error-IM: ImageMagick Error");
		log2file("ImageMagick Error: $out",TFT_ERROR);
		$lines = explode("\n",$out);
		for($i = 0; $i<count($lines); $i++) {
			header("X-TFT-Error-IM-Error-$i: " . trim($lines[$i]));
		}
		if ($output) readfile("images/default_error.gif");
	}
	return(true);
}

// createScaled($pic): creates a scaled for an specified image and saves it using GD.
// $pic: path to the picture
// $output: send jpeg to output
function createScaled_GD($pic,$new_width,$new_height,$size,$output = true, $force=false){
	global $data_directory, $album_directory, $file_permissions, $gd_available, $server_scaled_quality, $server_scaled_caching;
	$src = "$album_directory/$pic";
	$dest = getScaledFilePath($pic,$size);
	
	if (file_exists($dest))
	{
		if(filemtime($src) > filemtime($dest) or $force)
		{
			@unlink($dest);
		}
		else
		{
			return(false);
		}
	}
	
	// Open the picture and get it's size
	$imageinfo = GetImageSize($src);
	switch ($imageinfo[2]) {
		case 1: // GIF
			$image = imagecreatefromgif($src);
			break;
		case 2: // JPEG
			$image = imagecreatefromjpeg($src);
			break;
		case 3: // PNG
			$image = imagecreatefrompng($src);
			break;
		default:
			$image = null;
			break;
	}
	/*
	$image_x = imagesx($image);
	$image_y = imagesy($image);
	*/
	$image_x = $imageinfo[0];
	$image_y = $imageinfo[1];
	// create empty scaled and copy(resized) the picture
	if($gd_available >= 2){
		$scaled = imagecreatetruecolor($new_width, $new_height);
		imagecopyresampled($scaled, $image, 0, 0, 0, 0, $new_width, $new_height, $image_x, $image_y);
	}
	else {
		$scaled = imagecreate($new_width, $new_height); 
		imagecopyresized($scaled, $image, 0, 0, 0, 0, $new_width, $new_height, $image_x, $image_y);
	}

	if ($size==1 || $server_scaled_caching) {
		// save the scaled file
		imagejpeg($scaled, $dest, $server_scaled_quality);
		if (isset($file_permissions) && $file_permissions!='') chmod($dest,$file_permissions);
	}
	if ($output) imagejpeg($scaled, '', $server_scaled_quality);	
	imagedestroy($image);
	imagedestroy($scaled);
	
	return(true);
}

/* END SECTION:   scaler-functions
 */



/* BEGIN SECTION:   album-list-functions
 *
 *        build album lists for navigaion
 */

// navAlbums(): show albums to build the navi-bar
// $album_array: Array of album data from getAlbums()
// $level: used to display increasing indentation on recursive calls
function navAlbums($album_array, $parent_album='', $level=0) {
	global $pdf_only_for_admin, $zip_only_for_admin, $_SESSION;

	if(isset($_SESSION['admin'])) {
		$pdf = true;
		$zip = true;
	} else {
		$pdf = !($pdf_only_for_admin);
		$zip = !($zip_only_for_admin);
	}

/* added by saschalorenz 2005-03-14 */
$numberentries = 0;
	foreach($album_array as $album=>$album_info)
	{
		$album_dir=(empty($parent_album)? $album: $parent_album.'/'.$album);
		$album_name=basename($album);
		$url = makeURL("",array("album"=>$album_dir,"page"=>"1"));
	/* added by saschalorenz 2005-03-14 */
	$numberentries+=1;
	if($numberentries == 1) echo "<ul class=\"navi\">\n";
		if(isset($album_info['files']) and count($album_info['files'])>0) {
			echo "<li class='navi'><a href='$url'>$album_name</a> (".count($album_info['files']).") ";
			if($pdf) echo "<a href='pdf.php?album=$album_dir'><img src='images/pdf.gif' alt='PDF' border='0' /></a>";
			if($zip) echo "<a href='zip.php?album=$album_dir'><img src='images/zip.gif' alt='ZIP' border='0' /></a>";
		} else {
			echo "<li class='navi'><a href='$url'>$album</a>";
		}
		
		if(isset($album_info['subalbums']) and count($album_info['subalbums'])>0)
		{
			// call function again for subdirectory (recursive!), increasing the level
			navAlbums($album_info['subalbums'], $album_dir, $level+1);
		}
		echo "</li>\n";
	}
	if($numberentries>0) echo "</ul>\n";
}


// adminCacheAlbums(): show albums to build the navi-bar
// $album_array: Array of album data from getAlbums()
// $level: used to display increasing indentation on recursive calls
function adminCacheAlbums($album_array, $parent_album='', $level=0)
{
	global $data_directory;
	echo "<ul class='navi'>";
	foreach($album_array as $album=>$album_info)
	{
		$album_dir=(empty($parent_album)? $album: $parent_album.'/'.$album);
		$album_name=basename($album);
		if(isset($album_info['files']) and count($album_info['files'])>0) {
			$thumb_count=dirCountFiles('../'.$data_directory.'/thumbs/'.$album_dir);
			$scaled_count=dirCountFiles('../'.$data_directory.'/scaled/'.$album_dir);
			$url = makeURL('index.php',array('album'=>$album_dir,'action'=>'cache_admin','action2'=>'build_cache'));
			echo "<li class='navi'><a href='$url'>$album_name</a> ";
			echo "(".count($album_info['files'])." total, $thumb_count thumbnails, $scaled_count scaled images)\n";
		} else {
			$url = makeURL("",array("album"=>$album_dir,"page"=>"1"));
			echo "<li class='navi'><a href='$url'>$album</a>\n";
		}
		
		if(isset($album_info['subalbums']) and count($album_info['subalbums'])>0) {
			// call function again for subdirectory (recursive!), increasing the level
			adminCacheAlbums($album_info['subalbums'], $album_dir, $level+1);
		}
		echo "</li>";
	}
	echo "</ul>";
}


// dirCountFiles(): count the files in a directory
// $dir: directory to count
function dirCountFiles($dir)
{
	return(count(getFiles($dir)));
}

// getFiles(): get the files in a directory
// $dir: directory to get files for
function getFiles($dir)
{
	global $pic_extentions, $other_extentions;
	$file_list=array();
	
	if(is_dir($dir) and $fd = opendir($dir))
	{
		while($file = readdir($fd))
		{
			if(is_file("$dir/$file") && (preg_match($pic_extentions,$file) || preg_match($other_extentions,$file)))
			{
				$file_list[]="$dir/$file";
			}
		}
		closedir($fd);
	}
	
	return($file_list);
}

// getAlbums(): build album array to display navi-bar
// $album: begining album directory
function getAlbums($album = false){
	global $album_directory;
	global $pic_extentions, $other_extentions;
	$file_count=0;
	$dir = $album===false ? $album_directory : "$album_directory/$album";

	$albums=array();
	$fd = opendir($dir);
	while($file = readdir($fd)){
		if($file != "." && $file != ".." && $file != "CVS")
		{
			if(is_dir("$dir/$file"))
			{
				$folder = $album===false ? "$file" : "$album/$file";
				
				// call function again for subdirectory (recursive!)
				$albums[basename($folder)]=getAlbums("$folder");
			}
			elseif(is_file("$dir/$file") && (preg_match($pic_extentions,$file) || preg_match($other_extentions,$file)))
			{
				$files[]=$file;
			}
		}
	}
	closedir($fd);
	
	if($album===false)
	{
		return($albums);
	}
	
	
	$out=array();
	if(isset($albums) and count($albums) > 0)
		$out['subalbums']=$albums;
	if(isset($files) and count($files) > 0)
		$out['files']=$files;
	
	return($out);
				 
	// if(is_array($albums) and count($albums)>0)
		// return($albums);
	// else
		// return($file_count);
}

// @@@@@@@@@@@@@@@@@@@@@@@@@@@
// @@@@@@ TO BE DELETED @@@@@@
// @@@@@@@@@@@@@@@@@@@@@@@@@@@
// showNavigation(): show navi-bar
// $dir: base album directory
function showNavigation($album = false, $level = 0){
	global $album_directory;
	$dir = $album===false ? $album_directory : "$album_directory/$album";
//echo "$level '$album' '$dir'<br/>";
	$fd = opendir($dir);
	while($file = readdir($fd)){
		if($file != "." && $file != ".." && $file != "CVS" && is_dir("$dir/$file")) {
			$folder = $album===false ? "$file" : "$album/$file";
//echo "$level \"$folder\"<br/>";
			$url = makeURL("",array("album"=>"$folder","page"=>"1"));
			echo "<a class='menuEntry$level' href='$url'>$file</a><br/>\n";
			// call function again for subdirectory (recursive!), increasing the level
			showNavigation("$folder", $level + 1);
		}
	}
	closedir($fd);
}


/* END SECTION:   album-list-functions
 */
?>
